@@ -50,7 +50,7 @@
 #include "demux/demux.h"
 #include "stream/stream.h"
 #include "sub/dec_sub.h"
-#include "sub/find_subfiles.h"
+#include "external_files.h"
 #include "video/decode/dec_video.h"
 #include "video/out/vo.h"
 
@@ -198,10 +198,12 @@ void update_demuxer_properties(struct MPContext *mpctx)
                         continue;
                 }
             }
+            struct mp_log *log = mp_log_new(NULL, mpctx->log, "!display-tags");
             if (!had_output)
-                MP_INFO(mpctx, "File tags:\n");
-            MP_INFO(mpctx, " %s: %s\n", info->keys[n], info->values[n]);
+                mp_info(log, "File tags:\n");
+            mp_info(log, " %s: %s\n", info->keys[n], info->values[n]);
             had_output = true;
+            talloc_free(log);
         }
         talloc_free(mpctx->filtered_tags);
         mpctx->filtered_tags = info;
@@ -945,8 +947,13 @@ static void open_demux_thread(void *pctx)
         .stream_flags = args->stream_flags,
     };
     args->demux = demux_open_url(args->url, &p, args->cancel, global);
-    if (!args->demux && p.demuxer_failed)
-        args->err = MPV_ERROR_UNKNOWN_FORMAT;
+    if (!args->demux) {
+        if (p.demuxer_failed) {
+            args->err = MPV_ERROR_UNKNOWN_FORMAT;
+        } else {
+            args->err = MPV_ERROR_LOADING_FAILED;
+        }
+    }
     if (args->demux)
         args->tl = timeline_load(global, args->log, args->demux);
 }
@@ -1067,6 +1074,8 @@ static void play_current_file(struct MPContext *mpctx)
 
     mpctx->max_frames = opts->play_frames;
 
+    handle_force_window(mpctx, false);
+
     MP_INFO(mpctx, "Playing: %s\n", mpctx->filename);
 
 reopen_file:
@@ -1088,7 +1097,7 @@ reopen_file:
     }
 
     open_demux_reentrant(mpctx);
-    if (!mpctx->master_demuxer)
+    if (!mpctx->master_demuxer || mpctx->stop_play)
         goto terminate_playback;
     mpctx->demuxer = mpctx->master_demuxer;
 
@@ -1288,6 +1297,8 @@ terminate_playback:
     };
     mp_notify(mpctx, MPV_EVENT_END_FILE, &end_event);
 
+    MP_VERBOSE(mpctx, "finished playback, %s (reason %d)\n",
+               mpv_error_string(end_event.error), end_event.reason);
     if (mpctx->error_playing == MPV_ERROR_UNKNOWN_FORMAT)
         MP_ERR(mpctx, "Failed to recognize file format.\n");
     MP_INFO(mpctx, "\n");
